import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.util.SOSLogPublisher;
import com.kelvinluck.flickr.Flickr;
import com.kelvinluck.flickr.FlickrResponseListener;
import com.dynamicflash.utils.Delegate;
import com.kelvinluck.util.BasicButton;

/**
* Class: AuthDemo
*
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flickr.demo.AuthDemo 
{
	
	/**
	* Variable: apiKey
	* Your API Key.
	* 
	* See Also:
	* <http://www.flickr.com/services/api/misc.api_keys.html>
	**/
	var apiKey:String = "e6b54dde9c29528531034e258e745a6e";
	/**
	* Variable: apiSecret
	* Your flickr.com API secret
	* 
	* See Also:
	* <http://www.flickr.com/services/api/misc.api_keys.html>
	* 
	* <http://www.flickr.com/services/api/auth.spec.html>
	**/
	var apiSecret:String = "xxx";
	
	private var _flickr:Flickr;
	private var _target:MovieClip;
	private var _authListener:FlickrResponseListener;
	private var _authSharedObject:SharedObject;
	
	private var defaultTextFormat:TextFormat;
	private var loginScreen:MovieClip;
	private var loggedInScreen:MovieClip;
	private var waitForWebScreen:MovieClip;
	private var logoutScreen:MovieClip;
	
	/**
	* Function: AuthDemo
	* Constructor
	**/
	private function AuthDemo(target:MovieClip)
	{
		_target = target;
		LogWrapper.getInstance().init();
		LogWrapper.getLog().addPublisher(new SOSLogPublisher());
		
		defaultTextFormat = new TextFormat("arial");
		
		_flickr = Flickr.getFlickr();
		_flickr.apiKey = apiKey;
		_flickr.authSetSecret(apiSecret);
		
		_authListener = new FlickrResponseListener();
		_authListener.onAuthCheckToken = Delegate.create(this, onAuthCheckToken);
		_authListener.setSuppressOutput(true);
		_authSharedObject = SharedObject.getLocal("flickr_auth");
		if (_authSharedObject.data.token) {
			_flickr.authCheckToken(_authSharedObject.data.token);
		} else {
			displayPage("login");
		}
		
	}
	
	/**
	* Function: onAuthCheckToken
	* Called when there is a response from flickr.com to a call to
	* check a token.
	* 
	* Parameters:
	* token				-	The authentication token
	* 
	* See Also:
	* <FlickrResponseListener.onAuthCheckToken>
	*/
	
	function onAuthCheckToken(token:String)
	{
		_authSharedObject.data.token = token;
		_authSharedObject.flush();
		displayPage("loggedIn");
	}
	
	/**
	* Function: displayPage
	* Sets the screen up to display a given page.
	* 
	* Parameters:
	* page				-	The page to display ("login", "loggedIn" or "waitForWeb")
	**/
	function displayPage(page:String):Void
	{
		var screens:Array = [loginScreen, waitForWebScreen, loggedInScreen, logoutScreen];
		for (var i:Number=0; i<screens.length; i++) {
			screens[i]._visible = false;
		}
		switch (page) {
			case "login":
				if (typeof(loginScreen) != "movieclip") {
					loginScreen = _target.createEmptyMovieClip("loginScreen", 1);
					loginScreen.createTextField("intro", 1, 10, 10, 300, 10);
					loginScreen.intro.setNewTextFormat(defaultTextFormat);
					loginScreen.intro.wordWrap = true;
					loginScreen.intro.autoSize = true;
					loginScreen.intro.text = "You are not currently authenticated against flickr.com. Press the button below to authenticate.";
					var loginButton:BasicButton = new BasicButton(loginScreen, "loginBut", 3, "Login", {_x:20, _y:70});
					loginButton.onRelease = Delegate.create(this, onLoginClick);
				}
				loginScreen._visible = true;
				break;
			case "waitForWeb":
				if (typeof(waitForWebScreen) != "movieclip") {
					waitForWebScreen = _target.createEmptyMovieClip("waitForWebScreen", 2);
					waitForWebScreen.createTextField("intro", 1, 10, 10, 300, 10);
					waitForWebScreen.intro.setNewTextFormat(defaultTextFormat);
					waitForWebScreen.intro.wordWrap = true;
					waitForWebScreen.intro.autoSize = true;
					waitForWebScreen.intro.text = "This application requires your authentication before it can read your photos on flickr.com.\n\nPlease complete the authorisation process in the window which has popped up and then press the 'Continue' button below";
					var continueButton:BasicButton = new BasicButton(waitForWebScreen, "continueBut", 2, "Continue", {_x:120, _y:170});
					continueButton.onRelease = Delegate.create(this, onContinueClick);
					var cancelButton:BasicButton = new BasicButton(waitForWebScreen, "cancelBut", 3, "Cancel", {_x:20, _y:170});
					cancelButton.onRelease = Delegate.create(this, onCancelClick);
				}
				waitForWebScreen._visible = true;
				break;
			case "loggedIn":
				if (typeof(loggedInScreen) != "movieclip") {
					loggedInScreen = _target.createEmptyMovieClip("loggedInScreen", 3);
					loggedInScreen.createTextField("intro", 1, 10, 10, 300, 10);
					loggedInScreen.intro.setNewTextFormat(defaultTextFormat);
					loggedInScreen.intro.wordWrap = true;
					loggedInScreen.intro.autoSize = true;
					loggedInScreen.intro.text = "You are currently authenticated against flickr.com with the details below:\n\nUsername:" + _flickr.authUser.username + "\nPermissions:" + _flickr.authPerms;
					var logoutButton:BasicButton = new BasicButton(loggedInScreen, "logoutBut", 2, "Logout", {_x:10, _y:140});
					logoutButton.onRelease = Delegate.create(this, onLogoutClick);
				}
				loggedInScreen._visible = true;
				break;
			case "logout":
				if (typeof(logoutScreen) != "movieclip") {
					logoutScreen = _target.createEmptyMovieClip("loggedInScreen", 4);
					logoutScreen.createTextField("intro", 1, 10, 10, 300, 10);
					logoutScreen.intro.setNewTextFormat(defaultTextFormat);
					logoutScreen.intro.wordWrap = true;
					logoutScreen.intro.autoSize = true;
					logoutScreen.intro.text = "This instance of the application is now logged out. The application itself is still authenticated against flickr.com until you remove it from the 'Authentication list'";
					var okBut:BasicButton = new BasicButton(logoutScreen, "okBut", 2, "OK", {_x:10, _y:130});
					okBut.onRelease = Delegate.create(this, onLogoutOK);
				}
				logoutScreen._visible = true;
				break;
			default:
				LogWrapper.getLog().error("How can I display a page '"+page+"' when I don't know what it is?");
		}
	}
	
	/**
	* Function: onLoginClick
	**/
	function onLoginClick()
	{
		_flickr.authLogin("read");
		displayPage("waitForWeb");
	}
	/**
	* Function: onContinueClick
	**/
	function onContinueClick()
	{
		_flickr.authGetToken();
	}
	/**
	* Function: onCancelClick
	**/
	function onCancelClick()
	{
		displayPage("login");
	}
	/**
	* Function: onLogoutClick
	**/
	function onLogoutClick()
	{
		_flickr.authLogout();
		_authSharedObject.data.token = "";
		_authSharedObject.flush();
		displayPage("logout");
	}
	/**
	* Function: onLogoutOK
	**/
	function onLogoutOK()
	{
		displayPage("login");
	}
	
	/**
	* Function: main
	* Entry point into the application.
	**/
	public static function main() : Void 
	{
		var authDemo:AuthDemo = new AuthDemo(_root);
	}
	
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flickr.demo.AuthDemo]";
	}
}